import asyncio
import pyppeteer
import random
class browser:
    def __init__(self, url):
        self.url = url
        self.userAgent = "Mozilla/5.0 (iPhone; CPU iPhone OS 11_0 like Mac OS X) AppleWebKit/604.1.38 (KHTML, like Gecko) Version/11.0 Mobile/15A372 Safari/604.1"
        self.args = [
            "--no-sandbox",
            "--disable-setuid-sandbox",
            "--disable-infobars",
            "--window-position=0,0",
            "--ignore-certifcate-errors",
            "--ignore-certifcate-errors-spki-list",
            "--user-agent=" + self.userAgent
        ]

        self.options = {
        'args': self.args,
        'headless': True,
        'ignoreHTTPSErrors': True,
        'userDataDir': "./tmp"
        }

        asyncio.get_event_loop().run_until_complete(self.start())

    async def start(self):
        self.browser = await pyppeteer.launch(self.options)
        self.page = await self.browser.newPage()

        await self.page.emulate({'viewport': {
            'width':random.randint(320, 1920),
            'height':random.randint(320, 1920),
            'deviceScaleFactor':random.randint(1, 3),
            'isMobile':random.random() > 0.5,
            'hasTouch': random.random() > 0.5
        }})

        await self.page.setUserAgent(self.userAgent)

        await self.page.goto("https://www.tiktok.com/trending?lang=en", {
        'waitUntil': "load"
        })

        await self.page.evaluate('''() => {
          var b = {};
          for (let x of window.webpackJsonp) {
            if (typeof x[1]["duD4"] === "function") {
              x[1]["duD4"](null, b);
              break;
            }
          }

          if (typeof b.sign !== "function") {
            throw "No function found";
          }

          window.generateSignature = function generateSignature(url) {
              return b.sign({ url: url });
          };
          }''')
        
        self.signature = await self.sign(self.url)
        await self.browser.close()

    async def sign(self, string):
        res = await self.page.evaluate("generateSignature('" + string + "')")
        return res



import asyncio
import pyppeteer
import random
import requests
from .browser import browser
from bs4 import BeautifulSoup
import time
import json
from selenium import webdriver


class TikTokApi:
    #
    # The TikTokapi class constructor
    #
    def __init__(self, debug=False):
        if debug:
            print("Class initialized")

        self.referrer = "https://www.tiktok.com/@ondymikula/video/6756762109670477061"

    #
    # Method that retrives data from the api
    #
    def getData(self, api_url, signature, userAgent):
        url = api_url + \
            "&_signature=" + signature
        r = requests.get(url, headers={"method": "GET",
                                    "accept-encoding": "gzip, deflate, br",
                                    "Referer": self.referrer,
                                    "user-agent": userAgent,
                                    })           
        return r.json()


    #
    # Gets trending Tiktoks
    #
    def trending(self, count=30):
        api_url = "https://m.tiktok.com/api/item_list/?count={}&id=1&type=5&secUid=&maxCursor=1&minCursor=0&sourceType=12&appId=1233&verifyFp=".format(str(count))
        b = browser(api_url)

        return self.getData(api_url, b.signature, b.userAgent)['items']
        
    #
    # Gets a specific user's tiktoks
    #
    def userPosts(self, userID, secUID, count=30):
        api_url = "https://m.tiktok.com/api/item_list/?count={}&id={}&type=1&secUid={}&maxCursor=0&minCursor=0&sourceType=8&appId=1233&region=US&language=en&verifyFp=".format(str(count), str(userID), str(secUID))
        b = browser(api_url)
        return self.getData(api_url, b.signature, b.userAgent)['items']


    #
    # Gets tiktoks by music ID
    #
    # Note: not working for whatever reason. 
    #
    def bySound(self, id, count=30):
        api_url = "https://m.tiktok.com/share/item/list?secUid=&id={}&type=4&count={}&minCursor=0&maxCursor=0&shareUid=&lang=&verifyFp=".format(str(id), str(count))
        b = browser(api_url)
        return self.getData(api_url, b.signature, b.userAgent)['items']

    # 
    # Gets a user object for id and secUid
    #
    # Note: not working for whatever reason. 
    #
    def getUserObject(self, username):
        api_url = "https://m.tiktok.com/api/user/detail/?uniqueId={}&language=en&verifyFp=".format(username)
        b = browser(api_url)
        return self.getData(api_url, b.signature, b.userAgent)['userInfo']['user']

    
    #
    # Gets the source url of a given url for a tiktok
    #
    # video_url - the url of the video
    # return_bytes - 0 is just the url, 1 is the actual video bytes
    # chromedriver_path - path to your chrome driver executible
    #

    def get_Video_By_Url(self, video_url, return_bytes=0, chromedriver_path=None):
        if chromedriver_path != None:
            driver = webdriver.Chrome(executable_path=chromedriver_path)
        else:
            driver = webdriver.Chrome()
        driver.get(video_url)
        time.sleep(5)

        soup = BeautifulSoup(driver.page_source, 'html.parser')
        data = json.loads(soup.find_all('script', attrs={"id": "videoObject"})[0].text)

        if return_bytes == 0:
            return data['contentUrl']
        else:
            r = requests.get(data['contentUrl'])
            return r.content